@@ -4,20 +4,26 @@ import subprocess
 from pathlib import Path
 from typing import Any, List
 
-import typer
 from jinja2 import Environment, PackageLoader
 
+from . import logger
+from .exceptions import FetchError, GenerateError, GenerateScriptError
 from .fetcher import fetch
 from .parser import Blueprint
-from .settings import settings
 
-environment = Environment(loader=PackageLoader("ops2deb", "templates"))
+_environment = Environment(loader=PackageLoader("ops2deb", "templates"))
+
+
+def _format_command_output(output: str) -> str:
+    lines = output.splitlines()
+    output = "\n  ".join([line for line in lines])
+    return "> " + output
 
 
 class SourcePackage:
-    def __init__(self, blueprint: Blueprint):
+    def __init__(self, blueprint: Blueprint, work_directory: Path):
         self.directory_name = f"{blueprint.name}_{blueprint.version}_{blueprint.arch}"
-        self.output_directory = (settings.work_dir / self.directory_name).absolute()
+        self.output_directory = (work_directory / self.directory_name).absolute()
         self.debian_directory = self.output_directory / "debian"
         self.src_directory = self.output_directory / "src"
         self.tmp_directory = Path(f"/tmp/ops2deb_{self.directory_name}")
@@ -25,7 +31,7 @@ class SourcePackage:
         self.blueprint = blueprint.render(self.src_directory)
 
     def render_tpl(self, template_name: str) -> None:
-        template = environment.get_template(f"{template_name}.j2")
+        template = _environment.get_template(f"{template_name}.j2")
         package = self.blueprint.dict(exclude={"fetch", "script"})
         package.update({"version": self.debian_version})
         template.stream(package=package).dump(str(self.debian_directory / template_name))
@@ -40,33 +46,30 @@ class SourcePackage:
         for path in ["usr/bin", "usr/share", "usr/lib"]:
             (self.src_directory / path).mkdir(parents=True)
 
-    async def fetch(self) -> None:
+    async def fetch(self) -> "SourcePackage":
         if (remote_file := self.blueprint.fetch) is not None:
             await fetch(
                 url=remote_file.url,
                 expected_hash=remote_file.sha256,
                 save_path=self.tmp_directory,
             )
+        return self
 
     def generate(self) -> None:
-        typer.secho(
-            f"Generating source package {self.directory_name}...",
-            fg=typer.colors.BLUE,
-            bold=True,
-        )
+        logger.title(f"Generating source package {self.directory_name}...")
 
         # run script
         for line in self.blueprint.script:
-            typer.secho(f"$ {line}", fg=typer.colors.WHITE)
+            logger.info(f"$ {line}")
             result = subprocess.run(
                 line, shell=True, cwd=self.tmp_directory, capture_output=True
             )
             if stdout := result.stdout.decode():
-                typer.secho(stdout.strip(), fg=typer.colors.GREEN)
+                logger.info(_format_command_output(stdout))
             if stderr := result.stderr.decode():
-                typer.secho(stderr.strip(), fg=typer.colors.RED)
+                logger.error(_format_command_output(stderr))
             if result.returncode:
-                raise RuntimeError("Script failed")
+                raise GenerateScriptError
 
         # render debian/* files
         for template in [
@@ -80,8 +83,8 @@ class SourcePackage:
             self.render_tpl(template)
 
 
-def generate(blueprints: List[Blueprint]) -> None:
-    packages = [SourcePackage(b) for b in blueprints]
+def generate(blueprints: List[Blueprint], work_directory: Path) -> None:
+    packages = [SourcePackage(b, work_directory) for b in blueprints]
 
     # make sure we generate source packages in a clean environment
     # without artifacts from previous builds
@@ -90,13 +93,24 @@ def generate(blueprints: List[Blueprint]) -> None:
 
     # run fetch instructions (download, verify, extract) in parallel
     file_count = sum([1 for b in blueprints if b.fetch is not None])
-    typer.secho(f"Fetching {file_count} files...", fg=typer.colors.BLUE, bold=True)
+    logger.title(f"Fetching {file_count} files...")
 
-    async def fetch() -> Any:
-        return await asyncio.gather(*[p.fetch() for p in packages])
+    async def fetch_all() -> Any:
+        return await asyncio.gather(
+            *[p.fetch() for p in packages], return_exceptions=True
+        )
 
-    asyncio.run(fetch())
+    results = asyncio.run(fetch_all())
+
+    errors = [e for e in results if isinstance(e, Exception)]
+    for error in errors:
+        if not isinstance(error, FetchError):
+            raise error
 
     # run scripts, build debian/* files
+    packages = [p for p in results if isinstance(p, SourcePackage)]
     for package in packages:
         package.generate()
+
+    if errors:
+        raise GenerateError(f"{len(errors)} failures occurred")
