@@ -4,19 +4,26 @@ from typing import Any, Dict, Optional
 
 import httpx
 import ruamel.yaml
-import typer
 from pydantic import BaseModel
 from semver.version import Version
 
-from .fetcher import download
+from . import logger
+from .exceptions import Ops2debError, UpdaterError
+from .fetcher import download_file_to_cache
 from .parser import Blueprint, load, validate
-from .settings import settings
 
 
 class NewRelease(BaseModel):
+    name: str
     file_path: Path
     sha256: str
-    version: str
+    old_version: str
+    new_version: str
+
+
+def _error(msg: str) -> None:
+    logger.error(msg)
+    raise UpdaterError(msg)
 
 
 async def _bump_and_poll(
@@ -31,9 +38,11 @@ async def _bump_and_poll(
         if (remote_file := blueprint.render(version=str(version)).fetch) is None:
             break
         url = remote_file.url
-        if settings.verbose:
-            typer.secho(f"Trying {url}", fg=typer.colors.BRIGHT_BLACK)
-        response = await client.head(url)
+        logger.debug(f"Trying {url}")
+        try:
+            response = await client.head(url)
+        except httpx.HTTPError as e:
+            _error(f"Failed HEAD request to {url}. {str(e)}")
         status = response.status_code
         # FIXME: retry once on 500
         if status >= 400:
@@ -48,10 +57,7 @@ async def _find_latest_release(
 ) -> Optional[NewRelease]:
 
     if not Version.isvalid(blueprint.version):
-        typer.secho(
-            f"* {blueprint.name} is not using semantic versioning",
-            fg=typer.colors.YELLOW,
-        )
+        logger.warning(f"{blueprint.name} is not using semantic versioning")
         return None
 
     old_version = version = Version.parse(blueprint.version)
@@ -60,54 +66,75 @@ async def _find_latest_release(
         version = await _bump_and_poll(client, blueprint, version, True)
 
     if version != old_version:
-        typer.secho(
-            f"* {blueprint.name} can be bumped from {old_version} to {version}",
-            fg=typer.colors.WHITE,
-        )
+        logger.info(f"{blueprint.name} can be bumped from {old_version} to {version}")
 
-        file_path, sha256 = await download(
+        file_path, sha256 = await download_file_to_cache(
             blueprint.render(version=str(version)).fetch.url  # type: ignore
         )
         return NewRelease(
+            name=blueprint.name,
             file_path=file_path,
             sha256=sha256,
-            version=str(version),
+            old_version=blueprint.version,
+            new_version=str(version),
         )
 
     return None
 
 
-async def _update_blueprint_dict(blueprint_dict: Dict[str, Any]) -> bool:
+async def _update_blueprint_dict(blueprint_dict: Dict[str, Any]) -> Optional[NewRelease]:
     blueprint = Blueprint.parse_obj(blueprint_dict)
     if blueprint.fetch is None:
-        return True
+        return None
 
     release = await _find_latest_release(blueprint)
     if release is None:
-        return False
+        return None
 
-    blueprint_dict["version"] = release.version
+    blueprint_dict["version"] = release.new_version
     blueprint_dict["fetch"]["sha256"] = release.sha256
-    return True
+    if "revision" in blueprint_dict.keys():
+        blueprint_dict["revision"] = 1
+    return release
 
 
-def update(configuration_path: Path, dry_run: bool = False) -> bool:
+def update(
+    configuration_path: Path, dry_run: bool = False, output_path: Optional[Path] = None
+) -> None:
     yaml = ruamel.yaml.YAML()
     configuration_dict = load(configuration_path, yaml)
     validate(configuration_dict)
 
-    typer.secho("Looking for new releases...", fg=typer.colors.BLUE, bold=True)
+    logger.title("Looking for new releases...")
 
     async def run_tasks() -> Any:
         return await asyncio.gather(
-            *[_update_blueprint_dict(b) for b in configuration_dict]
+            *[_update_blueprint_dict(b) for b in configuration_dict],
+            return_exceptions=True,
         )
 
     results = asyncio.run(run_tasks())
+    new_releases = [r for r in results if isinstance(r, NewRelease)]
+    errors = [e for e in results if isinstance(e, Exception)]
 
-    if dry_run is False:
+    for error in errors:
+        if not isinstance(error, Ops2debError):
+            raise error
+
+    if dry_run is False and new_releases:
         with configuration_path.open("w") as output:
             yaml.dump(configuration_dict, output)
-        typer.secho("Configuration file updated", fg=typer.colors.BLUE, bold=True)
+        logger.info("Configuration file updated")
 
-    return bool(list(results))
+        if output_path is not None:
+            lines = [
+                f"Updated {r.name} from {r.old_version} to {r.new_version}"
+                for r in new_releases
+            ]
+            output_path.write_text("\n".join(lines))
+
+    if not new_releases:
+        logger.info("Did not found any updates")
+
+    if errors:
+        raise UpdaterError(f"{len(errors)} failures occurred")
